import { AppInfo } from "@v-act/vjs.framework.extension.platform.data.manager.runtime.info";
import { ExpressionEngine } from "@v-act/vjs.framework.extension.platform.engine.expression";
import { ExceptionFactory } from "@v-act/vjs.framework.extension.platform.interface.exception";
import { DatasourceFactory } from "@v-act/vjs.framework.extension.platform.interface.model.datasource";
import { ScopeManager } from "@v-act/vjs.framework.extension.platform.interface.scope";
import { StringUtil } from "@v-act/vjs.framework.extension.util.string";
const execute = function(params) {
  let functionName = params.functionName, context = params.context, args = params.args;
  const functionDef = AppInfo.getFuncDefine(functionName);
  if (!functionDef) {
    let exception = ExceptionFactory.create({
      message: "\u51FD\u6570\u3010" + functionName + "\u3011\u4E0D\u5B58\u5728\uFF0C\u8BF7\u68C0\u67E5\u662F\u5426\u5DF2\u90E8\u7F72\u5BF9\u5E94\u51FD\u6570\uFF0C\u5E76\u6253\u5305\u5230functionLib\u6A21\u5757\u4E2D\u3002",
      exceptionDatas: genExceptionData(context && context.args || "", functionName),
      type: ExceptionFactory.TYPES.System
    });
    throw exception;
  }
  let mainFunc = functionDef.main;
  if (mainFunc) {
    let mainFuncArgs = args ? args : [context];
    try {
      return exeExtFunc({ mainFunc, funcContext: mainFuncArgs[0] });
    } catch (e) {
      if (ExceptionFactory.isException(e)) {
        throw e;
      } else {
        let exception = ExceptionFactory.create({
          message: "\u51FD\u6570\u3010" + functionName + "\u3011\u6267\u884C\u5931\u8D25\uFF0C\u9519\u8BEF\u539F\u56E0\uFF1A" + e.message,
          type: ExceptionFactory.getExceptionTypeByError(e, ExceptionFactory.TYPES.System),
          exceptionDatas: genExceptionData(context && context.args || "", functionName),
          error: e
        });
        throw exception;
      }
    }
  } else {
    let exception = ExceptionFactory.create({
      message: "\u51FD\u6570\u3010" + functionName + "\u3011\u7684\u5B9E\u73B0\u4E0D\u7B26\u5408\u89C4\u8303\uFF0C\u6CA1\u6709\u5B9E\u73B0\u4E3B\u5165\u53E3main\u65B9\u6CD5\uFF0C\u8BF7\u68C0\u67E5\u5185\u90E8\u903B\u8F91\u3002",
      type: ExceptionFactory.TYPES.System,
      exceptionDatas: genExceptionData(context && context.args || "", functionName),
      error: ""
    });
    throw exception;
  }
};
let genExceptionData = function(args, functionName) {
  let scope = ScopeManager.getScope();
  let componentCode = scope.getComponentCode();
  let windowCode = ScopeManager.isWindowScope(scope.getInstanceId()) ? scope.getWindowCode() : null;
  return [
    {
      name: "\u6784\u4EF6\u7F16\u7801",
      code: "componentCode",
      value: componentCode
    },
    {
      name: "\u7A97\u4F53\u7F16\u7801",
      code: "windowCode",
      value: windowCode
    },
    {
      name: "\u51FD\u6570\u7F16\u7801",
      code: "functionCode",
      value: functionName
    },
    {
      name: "\u51FD\u6570\u53C2\u6570",
      code: "functionParams",
      value: args
    }
  ];
};
function exeExtFunc(params) {
  const funcCtx = params.funcContext;
  const args = funcCtx.getArgs();
  const mainFunc = params.mainFunc;
  const rs = typeof mainFunc == "string" ? StringUtil.getNamespaceDef(mainFunc) : { func: mainFunc, caller: mainFunc };
  if (!rs) {
    throw new Error("\u672A\u627E\u5230\u4E3B\u5165\u53E3\u51FD\u6570");
  }
  if (args && args.length > 0 && typeof window != "undefined" && window.vds && window.vds.ds && typeof window.vds.ds._genDatasourceByDs == "function") {
    for (var i = 0, len = args.length; i < len; i++) {
      var arg = args[i];
      if (arg && typeof arg == "object" && DatasourceFactory.isDatasource(arg)) {
        args.splice(i, 1, window.vds.ds._genDatasourceByDs(arg));
      }
    }
  }
  ExpressionEngine._setThreadFunContext(funcCtx);
  try {
    var result = rs.func.apply(rs.caller, args);
    ExpressionEngine._clearThreadFunContext();
    return result;
  } finally {
    ExpressionEngine._clearThreadFunContext();
  }
}
var FunctionEngine = /* @__PURE__ */ Object.freeze({
  __proto__: null,
  [Symbol.toStringTag]: "Module",
  exeExtFunc,
  execute
});
export { FunctionEngine };
